@@ -7,7 +7,7 @@ import numpy as np
 from langchain_core.prompts import ChatPromptTemplate, MessagesPlaceholder
 from langchain_core.runnables.base import Runnable
 from langgraph.checkpoint.memory import MemorySaver
-from langgraph.graph import StateGraph
+from langgraph.graph import StateGraph, END
 from langgraph.graph.graph import CompiledGraph
 from sqlalchemy.ext.asyncio import AsyncSession
 
@@ -15,6 +15,7 @@ from nlpbridge import ChatAgent
 from nlpbridge.persistent.db_crud import CRUDRouter, CRUDNode, CRUDEdge, CRUDTemplate
 from nlpbridge.persistent.mysql_dataschema import Node as NodeModel
 from nlpbridge.text.tools import tool_list
+from nlpbridge.config import CONFIG
 
 memory = MemorySaver()
 
@@ -82,6 +83,7 @@ class Node(Callable[[dict], TypedDict]):
     def rand_prompt(self):
         # start_sys_template + user_prompt + end_sys_template
         start_sys_template, end_sys_template = get_sys_template_pair(self.system_templates)
+
         start_sys_prompt = start_sys_template.content if start_sys_template else ""
         end_sys_prompt = end_sys_template.content if end_sys_template else ""
 
@@ -92,9 +94,7 @@ class Node(Callable[[dict], TypedDict]):
             else:
                 # chat_limit - len(user_templates), 随机选择一个user_template
                 user_template = random.choice(self.user_templates)
-        # meta_prompt = """
-        # {tmp1}{tmp2}....
-        # """
+
         user_prompt = user_template.content if user_template else ""
         return ChatPromptTemplate.from_messages(
             [
@@ -118,8 +118,8 @@ class Node(Callable[[dict], TypedDict]):
         # question = input("[Question]: ")
         question = "你好呀，你是谁？"
         response = self.agent.run(question)
-        print(f"[current agent generate]: {response}")
-        print(f"[node state]:\n{state}")
+        # print(f"[current agent generate]: {response}")
+        # print(f"[node state]:\n{state}")
         return {"context": {"input": question, "response": response.content}}
 
     @classmethod
@@ -186,6 +186,8 @@ class TemplateManager:
 class GraphState(TypedDict):
     nodes: Optional[List[Node]]
     context: Any | Dict[str, Any]
+    jump_points: Optional[List[str]]
+    end_points: Optional[List[str]]
 
 
 class Condition(Callable[[dict], TypedDict]):
@@ -195,7 +197,6 @@ class Condition(Callable[[dict], TypedDict]):
 
     def call(self, node_state: TypedDict) -> str:
         # Randomly select the next node from the current node and the node pointed to by the node.
-        # print("Condition-------", node_state)
 
         # 如果到graph的末端节点, 此时target node只剩下1个, 就直接返回自己; 并且如果chat_limit<=0就终止
         if len(self.target_nodes[:-1]) < 2:
@@ -219,7 +220,7 @@ class Condition(Callable[[dict], TypedDict]):
         if node_state.get("stop", False):
             logger.debug(f"stop signal received")
             # return "END"
-            return self.target_nodes[-1].name
+            return self.target_nodes[-2].name
 
         return self.call(node_state)
 
@@ -280,17 +281,19 @@ class RouterManager:
         template_ids = self.get_template_ids(nodes)
         templates = await TemplateManager.get_templates(template_ids=template_ids, db_session=db_session)
 
-        print(f'router: {router}')
-        print(f'meta_template_id: {router.meta_template_id}')
-
         # Add basic info template
-        if router.meta_template_id != None:
+        if router.meta_template_id != None and router.meta_template_id != '':
             meta_template_id = int(router.meta_template_id)
-            basic_info_template = await TemplateManager.get_templates([meta_template_id])[0]
-            print(f'basic info: {basic_info_template.content}') # debug
-            for tmp in templates:
-                tmp.content = f'\n{basic_info_template.content}\n{tmp.content}'
-                print(f'template info: {tmp.content}') # debug
+            basic_info_template = await TemplateManager.get_templates([meta_template_id])
+            basic_info_template = basic_info_template[0]
+
+            for index, tmp in enumerate(templates):
+                if index == 0:
+                    tmp.content = f'\n---------------\n{tmp.content}\n\n{basic_info_template.content}\n'
+                elif index == len(templates) - 1:
+                    continue
+                else:
+                    tmp.content = f'---------------\n{tmp.content}\n'
 
         # Convert nodes to runnable nodes
         runnable_nodes = self.convert_to_runnable_node(nodes, edges, templates, llm, chat_id, node_cls, **kwargs)
@@ -303,7 +306,24 @@ class RouterManager:
         graph = self._create_graph(runnable_nodes, edges, condition_cls)
         uuid = str(uuid4())
         cfg = {"configurable": {"thread_id": uuid}, "recursion_limit": recursion_limit}
-        graph.update_state(config=cfg, values={"nodes": nodes})
+
+        jump_point_id = CONFIG.llm.jump_points_id
+        jump_points = await TemplateManager.get_templates([jump_point_id])
+        if len(jump_points) != 0:
+            jump_points = jump_points[0]
+            jump_points = jump_points.content.split(',')
+        else:
+            jump_points = None
+
+        end_point_id = CONFIG.llm.end_points_id
+        end_points = await TemplateManager.get_templates([end_point_id])
+        if len(end_points) != 0:
+            end_points = end_points[0]
+            end_points = end_points.content.split(',')
+        else:
+            end_points = None
+
+        graph.update_state(config=cfg, values={"nodes": nodes, "jump_points": jump_points, "end_points": end_points})
 
         return graph, cfg
 
@@ -344,8 +364,9 @@ class RouterManager:
             path_map[cur_node_name] = cur_node_name
             # add node:END to the all nodes:
             # TODO: replace END to Default node.
-            # path_map["END"] = END
-            # target_nodes.append(END)
+            path_map["end"] = END
+            path_map["END"] = END
+            target_nodes.append(END)
 
             graph.add_conditional_edges(
                 cur_node_name,
